@@ -85,9 +85,9 @@ class OVNClient(object):
     def _transaction(self, commands, txn=None):
         """Create a new transaction or add the commands to an existing one."""
         if txn is None:
-            with self._nb_idl.transaction(check_error=True) as txn:
+            with self._nb_idl.transaction(check_error=True) as new_txn:
                 for cmd in commands:
-                    txn.add(cmd)
+                    new_txn.add(cmd)
         else:
             for cmd in commands:
                 txn.add(cmd)
@@ -204,7 +204,7 @@ class OVNClient(object):
             port_type = 'vtep'
             options = {'vtep-physical-switch': vtep_physical_switch,
                        'vtep-logical-switch': vtep_logical_switch}
-            addresses = [ovn_const.UNKNOWN_ADDR]
+            addresses = ["unknown"]
             parent_name = []
             tag = []
             port_security = []
@@ -227,21 +227,19 @@ class OVNClient(object):
             addresses = [address]
             addresses.extend(new_macs)
 
-            # Only adjust the OVN type if the port is not owned by Neutron
-            # DHCP agents.
-            if (port['device_owner'] == const.DEVICE_OWNER_DHCP and
-                    not utils.is_neutron_dhcp_agent_port(port)):
-                port_type = 'localport'
-
-            # The "unknown" address should only be set for the normal LSP
-            # ports (the ones which type is empty)
-            if not port_security and not port_type:
+            if not port_security:
                 # Port security is disabled for this port.
                 # So this port can send traffic with any mac address.
                 # OVN allows any mac address from a port if "unknown"
                 # is added to the Logical_Switch_Port.addresses column.
                 # So add it.
-                addresses.append(ovn_const.UNKNOWN_ADDR)
+                addresses.append("unknown")
+
+            # Only adjust the OVN type if the port is not owned by Neutron
+            # DHCP agents.
+            if (port['device_owner'] == const.DEVICE_OWNER_DHCP and
+                    not port['device_id'].startswith('dhcp')):
+                port_type = 'localport'
 
         dhcpv4_options = self._get_port_dhcp_options(port, const.IP_VERSION_4)
         dhcpv6_options = self._get_port_dhcp_options(port, const.IP_VERSION_6)
@@ -302,33 +300,19 @@ class OVNClient(object):
             # The lport_name *must* be neutron port['id'].  It must match the
             # iface-id set in the Interfaces table of the Open_vSwitch
             # database which nova sets to be the port ID.
-
-            kwargs = {
-                'lport_name': port['id'],
-                'lswitch_name': lswitch_name,
-                'addresses': port_info.addresses,
-                'external_ids': external_ids,
-                'parent_name': port_info.parent_name,
-                'tag': port_info.tag,
-                'enabled': port.get('admin_state_up'),
-                'options': port_info.options,
-                'type': port_info.type,
-                'port_security': port_info.port_security,
-                'dhcpv4_options': dhcpv4_options,
-                'dhcpv6_options': dhcpv6_options
-            }
-
-            # NOTE(mjozefcz): Do not set addresses if the port is not
-            # bound and has no device_owner - possibly it is a VirtualIP
-            # port used for Octavia (VRRP).
-            # For more details check related bug #1789686.
-            if (not port.get('device_owner') and
-                port.get(portbindings.VIF_TYPE) ==
-                    portbindings.VIF_TYPE_UNBOUND):
-                kwargs['addresses'] = []
-
             port_cmd = txn.add(self._nb_idl.create_lswitch_port(
-                **kwargs))
+                lport_name=port['id'],
+                lswitch_name=lswitch_name,
+                addresses=port_info.addresses,
+                external_ids=external_ids,
+                parent_name=port_info.parent_name,
+                tag=port_info.tag,
+                enabled=port.get('admin_state_up'),
+                options=port_info.options,
+                type=port_info.type,
+                port_security=port_info.port_security,
+                dhcpv4_options=dhcpv4_options,
+                dhcpv6_options=dhcpv6_options))
 
             # Handle ACL's for this port. If we're not using Port Groups
             # because either the schema doesn't support it or we didn't
@@ -433,16 +417,6 @@ class OVNClient(object):
                 dhcpv6_options = txn.add(port_info.dhcpv6_options['cmd'])
             else:
                 dhcpv6_options = [port_info.dhcpv6_options['uuid']]
-
-            # NOTE(mjozefcz): Do not set addresses if the port is not
-            # bound and has no device_owner - possibly it is a VirtualIP
-            # port used for Octavia (VRRP).
-            # For more details check related bug #1789686.
-            if (not port.get('device_owner') and
-                port.get(portbindings.VIF_TYPE) ==
-                    portbindings.VIF_TYPE_UNBOUND):
-                columns_dict['addresses'] = []
-
             # NOTE(lizk): Fail port updating if port doesn't exist. This
             # prevents any new inserted resources to be orphan, such as port
             # dhcp options or ACL rules for port, e.g. a port was created
@@ -627,7 +601,9 @@ class OVNClient(object):
         context = n_context.get_admin_context()
         fip_db = self._l3_plugin._get_floatingip(context, floatingip['id'])
 
+        func = self._nb_idl.add_nat_rule_in_lrouter
         gw_lrouter_name = utils.ovn_name(router_id)
+        nat_rule_args = (gw_lrouter_name,)
         # TODO(chandrav): Since the floating ip port is not
         # bound to any chassis, packets destined to floating ip
         # will be dropped. To overcome this, delete the floating
@@ -638,6 +614,21 @@ class OVNClient(object):
                         fip_db['floating_port_id'],
                         utils.ovn_name(floatingip['floating_network_id'])))
 
+        # Get the list of nat rules and check if the external_ip
+        # with type 'dnat_and_snat' already exists or not.
+        # If exists, set the new value.
+        # This happens when the port associated to a floating ip
+        # is deleted before the disassociation.
+        lrouter_nat_rules = self._nb_idl.get_lrouter_nat_rules(
+            gw_lrouter_name)
+        for nat_rule in lrouter_nat_rules:
+            if (nat_rule['external_ip'] ==
+                    floatingip['floating_ip_address'] and
+                    nat_rule['type'] == 'dnat_and_snat'):
+                func = self._nb_idl.set_nat_rule_in_lrouter
+                nat_rule_args = (gw_lrouter_name, nat_rule['uuid'])
+                break
+
         ext_ids = {
             ovn_const.OVN_FIP_EXT_ID_KEY: floatingip['id'],
             ovn_const.OVN_REV_NUM_EXT_ID_KEY: str(utils.get_revision_number(
@@ -654,36 +645,14 @@ class OVNClient(object):
             columns['logical_port'] = floatingip['port_id']
             ext_ids[ovn_const.OVN_FIP_EXT_MAC_KEY] = port['mac_address']
             if self._nb_idl.lsp_get_up(floatingip['port_id']).execute():
-                port_net = self._plugin.get_network(context,
-                                                    port['network_id'])
-                if port_net.get(pnet.NETWORK_TYPE) != const.TYPE_VLAN:
-                    columns['external_mac'] = port['mac_address']
+                columns['external_mac'] = port['mac_address']
 
         # TODO(dalvarez): remove this check once the minimum OVS required
         # version contains the column (when OVS 2.8.2 is released).
         if self._nb_idl.is_col_present('NAT', 'external_ids'):
             columns['external_ids'] = ext_ids
 
-        commands.append(self._nb_idl.add_nat_rule_in_lrouter(gw_lrouter_name,
-                                                             **columns))
-
-        # Get the logical port (of the private network) and set the field
-        # external_ids:fip=<FIP>. This will be used by the ovn octavia driver
-        # to add the floating ip as vip in the Load_Balancer.vips column.
-        private_lsp = self._nb_idl.get_lswitch_port(floatingip['port_id'])
-
-        if private_lsp:
-            port_fip = {
-                ovn_const.OVN_PORT_FIP_EXT_ID_KEY:
-                    floatingip['floating_ip_address']}
-            commands.append(
-                self._nb_idl.db_set('Logical_Switch_Port', private_lsp.uuid,
-                                    ('external_ids', port_fip))
-            )
-        else:
-            LOG.warning("LSP for floatingip %s, has not been found! "
-                        "Cannot set FIP on VIP.",
-                        floatingip['id'])
+        commands.append(func(*nat_rule_args, **columns))
         self._transaction(commands, txn=txn)
 
     def _delete_floatingip(self, fip, lrouter, txn=None):
@@ -691,20 +660,6 @@ class OVNClient(object):
                     lrouter, type='dnat_and_snat',
                     logical_ip=fip['logical_ip'],
                     external_ip=fip['external_ip'])]
-        try:
-            port_id = (
-                fip['external_ids'].get(ovn_const.OVN_FIP_PORT_EXT_ID_KEY))
-            if port_id:
-                private_lsp = self._nb_idl.get_lswitch_port(port_id)
-                if private_lsp:
-                    commands.append(
-                        self._nb_idl.db_remove(
-                            'Logical_Switch_Port', private_lsp.uuid,
-                            'external_ids',
-                            (ovn_const.OVN_PORT_FIP_EXT_ID_KEY))
-                    )
-        except KeyError:
-            LOG.debug("FIP %s doesn't have external_ids.", fip)
         self._transaction(commands, txn=txn)
 
     def update_floatingip_status(self, floatingip):
@@ -755,10 +710,15 @@ class OVNClient(object):
             floatingip['id'], floatingip, ovn_const.TYPE_FLOATINGIPS)
         with self._nb_idl.transaction(check_error=True) as txn:
             txn.add(check_rev_cmd)
-            if ovn_fip:
+            if (ovn_fip and
+                (floatingip['fixed_ip_address'] != ovn_fip['logical_ip'] or
+                 floatingip['port_id'] != ovn_fip['external_ids'].get(
+                    ovn_const.OVN_FIP_PORT_EXT_ID_KEY))):
+
                 lrouter = ovn_fip['external_ids'].get(
                     ovn_const.OVN_ROUTER_NAME_EXT_ID_KEY,
                     utils.ovn_name(router_id))
+
                 self._delete_floatingip(ovn_fip, lrouter, txn=txn)
                 fip_status = const.FLOATINGIP_STATUS_DOWN
 
@@ -1132,21 +1092,17 @@ class OVNClient(object):
                   candidates)
         return candidates
 
-    def _get_physnet(self, net_id):
-        extnet = self._plugin.get_network(n_context.get_admin_context(),
-                                          net_id)
-        if extnet.get(pnet.NETWORK_TYPE) in [const.TYPE_FLAT,
-                                             const.TYPE_VLAN]:
-            return extnet.get(pnet.PHYSICAL_NETWORK)
+    def _get_physnet(self, network):
+        if network.get(pnet.NETWORK_TYPE) in [const.TYPE_FLAT,
+                                              const.TYPE_VLAN]:
+            return network.get(pnet.PHYSICAL_NETWORK)
 
     def _gen_router_port_ext_ids(self, port):
         ext_ids = {
             ovn_const.OVN_REV_NUM_EXT_ID_KEY: str(utils.get_revision_number(
                 port, ovn_const.TYPE_ROUTER_PORTS)),
             ovn_const.OVN_SUBNET_EXT_IDS_KEY:
-                ' '.join(utils.get_port_subnet_ids(port)),
-            ovn_const.OVN_NETWORK_NAME_EXT_ID_KEY:
-                utils.ovn_name(port['network_id'])}
+                ' '.join(utils.get_port_subnet_ids(port))}
 
         router_id = port.get('device_id')
         if router_id:
@@ -1164,8 +1120,18 @@ class OVNClient(object):
         is_gw_port = const.DEVICE_OWNER_ROUTER_GW == port.get(
             'device_owner')
         columns = {}
+        port_net = self._plugin.get_network(n_context.get_admin_context(),
+                                            port['network_id'])
+        # For VLAN type networks we need to set the
+        # "reside-on-redirect-chassis" option so the routing for this
+        # logical router port is centralized in the chassis hosting the
+        # distributed gateway port.
+        # https://github.com/openvswitch/ovs/commit/85706c34d53d4810f54bec1de662392a3c06a996
+        if port_net.get(pnet.NETWORK_TYPE) == const.TYPE_VLAN:
+            columns['options'] = {'reside-on-redirect-chassis': 'true'}
+
         if is_gw_port:
-            physnet = self._get_physnet(port['network_id'])
+            physnet = self._get_physnet(port_net)
             candidates = self.get_candidates_for_scheduling(physnet)
             selected_chassis = self._ovn_scheduler.select(
                 self._nb_idl, self._sb_idl, lrouter_port_name,
@@ -1176,18 +1142,6 @@ class OVNClient(object):
         lsp_address = ovn_const.DEFAULT_ADDR_FOR_LSP_WITH_PEER
         if ipv6_ra_configs:
             columns['ipv6_ra_configs'] = ipv6_ra_configs
-            # TODO(numans)
-            # If we set router port's peer logical switch port with
-            # addresses=ovn_const.DEFAULT_ADDRESS_FOR_LSP_WITH_PEER_ROUTER_PORT
-            # ovn-northd is adding IPv6 Neighbor Adv flow in the logical switch
-            # pipeline using the 'nd_na' action.
-            # 'nd_na' action doesn't set the Router bit in the 'flags' field
-            # of the Neigh Adv packet in response to the Neigh Solicitation
-            # packet for the router IPv6 address. See bug #1788684.
-            # Until it is fixed in core OVN, set lsp_address to the mac_address
-            # of port so that we don't set addresses=router. Once that is fixed
-            # remove the below line.
-            lsp_address = [port['mac_address']]
 
         commands = [
             self._nb_idl.add_lrouter_port(
@@ -1245,20 +1199,6 @@ class OVNClient(object):
                 port['fixed_ips']))
 
         lsp_address = ovn_const.DEFAULT_ADDR_FOR_LSP_WITH_PEER
-        if ipv6_ra_configs:
-            # TODO(numans)
-            # If we set router port's peer logical switch port with
-            # addresses=ovn_const.DEFAULT_ADDRESS_FOR_LSP_WITH_PEER_ROUTER_PORT
-            # ovn-northd is adding IPv6 Neighbor Adv flow in the logical switch
-            # pipeline using the 'nd_na' action.
-            # 'nd_na' action doesn't set the Router bit in the 'flags' field
-            # of the Neigh Adv packet in response to the Neigh Solicitation
-            # packet for the router IPv6 address. See bug #1788684.
-            # Until it is fixed in core OVN, set lsp_address to the mac_address
-            # of port so that we don't set addresses=router. Once that is fixed
-            # remove this if condition.
-            lsp_address = [port['mac_address']]
-
         lrp_name = utils.ovn_lrouter_port_name(port['id'])
         update = {'networks': networks, 'ipv6_ra_configs': ipv6_ra_configs}
         is_gw_port = const.DEVICE_OWNER_ROUTER_GW == port.get(
@@ -1377,16 +1317,15 @@ class OVNClient(object):
         txn.add(self._nb_idl.create_lswitch_port(
             lport_name=utils.ovn_provnet_port_name(network['id']),
             lswitch_name=utils.ovn_name(network['id']),
-            addresses=[ovn_const.UNKNOWN_ADDR],
+            addresses=['unknown'],
             external_ids={},
-            type=ovn_const.LSP_TYPE_LOCALNET,
+            type='localnet',
             tag=tag if tag else [],
             options={'network_name': physnet}))
 
     def _gen_network_external_ids(self, network):
         ext_ids = {
             ovn_const.OVN_NETWORK_NAME_EXT_ID_KEY: network['name'],
-            ovn_const.OVN_NETWORK_MTU_EXT_ID_KEY: str(network['mtu']),
             ovn_const.OVN_REV_NUM_EXT_ID_KEY: str(
                 utils.get_revision_number(network, ovn_const.TYPE_NETWORKS))}
 
@@ -1479,19 +1418,7 @@ class OVNClient(object):
         with self._nb_idl.transaction(check_error=True) as txn:
             txn.add(check_rev_cmd)
             ext_ids = self._gen_network_external_ids(network)
-            lswitch = self._nb_idl.get_lswitch(lswitch_name)
             txn.add(self._nb_idl.set_lswitch_ext_ids(lswitch_name, ext_ids))
-            # Check if previous mtu is different than current one,
-            # checking will help reduce number of operations
-            if (not lswitch or
-                    lswitch.external_ids.get(
-                        ovn_const.OVN_NETWORK_MTU_EXT_ID_KEY) !=
-                    str(network['mtu'])):
-                context = n_context.get_admin_context()
-                subnets = self._plugin.get_subnets_by_network(
-                    context, network['id'])
-                for subnet in subnets:
-                    self.update_subnet(subnet, network, txn)
 
         if check_rev_cmd.result == ovn_const.TXN_COMMITTED:
             if qos_update_required:
@@ -1531,6 +1458,29 @@ class OVNClient(object):
 
         return dhcp_options
 
+    def _process_global_dhcp_opts(self, options, ip_version):
+        if ip_version == 4:
+            global_options = config.get_global_dhcpv4_opts()
+        else:
+            global_options = config.get_global_dhcpv6_opts()
+
+        for option, value in global_options.items():
+            if option in ovn_const.GLOBAL_DHCP_OPTS_BLACKLIST[ip_version]:
+                # This option is not allowed to be set with a global setting
+                LOG.debug('DHCP option %s is not permitted to be set in '
+                          'global options. This option will be ignored.')
+                continue
+            # If the value is null (i.e. config ntp_server:), treat it as
+            # a request to remove the option
+            if value:
+                options[option] = value
+            else:
+                try:
+                    del(options[option])
+                except KeyError:
+                    # Option not present, job done
+                    pass
+
     def _get_ovn_dhcpv4_opts(self, subnet, network, server_mac=None):
         metadata_port_ip = self._find_metadata_port_ip(
             n_context.get_admin_context(), subnet)
@@ -1588,6 +1538,8 @@ class OVNClient(object):
 
             options['classless_static_route'] = '{' + ', '.join(routes) + '}'
 
+        self._process_global_dhcp_opts(options, ip_version=4)
+
         return options
 
     def _get_ovn_dhcpv6_opts(self, subnet, server_id=None):
@@ -1605,6 +1557,8 @@ class OVNClient(object):
         if subnet.get('ipv6_address_mode') == const.DHCPV6_STATELESS:
             dhcpv6_opts[ovn_const.DHCPV6_STATELESS_OPT] = 'true'
 
+        self._process_global_dhcp_opts(dhcpv6_opts, ip_version=6)
+
         return dhcpv6_opts
 
     def _remove_subnet_dhcp_options(self, subnet_id, txn):
@@ -1713,15 +1667,7 @@ class OVNClient(object):
             self._add_subnet_dhcp_options(subnet, network)
         db_rev.bump_revision(subnet, ovn_const.TYPE_SUBNETS)
 
-    def _modify_subnet_dhcp_options(self, subnet, ovn_subnet, network, txn):
-        if subnet['enable_dhcp'] and not ovn_subnet:
-            self._enable_subnet_dhcp_options(subnet, network, txn)
-        elif subnet['enable_dhcp'] and ovn_subnet:
-            self._update_subnet_dhcp_options(subnet, network, txn)
-        elif not subnet['enable_dhcp'] and ovn_subnet:
-            self._remove_subnet_dhcp_options(subnet['id'], txn)
-
-    def update_subnet(self, subnet, network, txn=None):
+    def update_subnet(self, subnet, network):
         ovn_subnet = self._nb_idl.get_subnet_dhcp_options(
             subnet['id'])['subnet']
 
@@ -1731,13 +1677,15 @@ class OVNClient(object):
 
         check_rev_cmd = self._nb_idl.check_revision_number(
             subnet['id'], subnet, ovn_const.TYPE_SUBNETS)
-        if not txn:
-            with self._nb_idl.transaction(check_error=True) as txn_n:
-                txn_n.add(check_rev_cmd)
-                self._modify_subnet_dhcp_options(subnet, ovn_subnet, network,
-                                                 txn_n)
-        else:
-            self._modify_subnet_dhcp_options(subnet, ovn_subnet, network, txn)
+        with self._nb_idl.transaction(check_error=True) as txn:
+            txn.add(check_rev_cmd)
+            if subnet['enable_dhcp'] and not ovn_subnet:
+                self._enable_subnet_dhcp_options(subnet, network, txn)
+            elif not subnet['enable_dhcp'] and ovn_subnet:
+                self._remove_subnet_dhcp_options(subnet['id'], txn)
+            elif subnet['enable_dhcp'] and ovn_subnet:
+                self._update_subnet_dhcp_options(subnet, network, txn)
+
         if check_rev_cmd.result == ovn_const.TXN_COMMITTED:
             db_rev.bump_revision(subnet, ovn_const.TYPE_SUBNETS)
 
@@ -1774,11 +1722,11 @@ class OVNClient(object):
         with self._nb_idl.transaction(check_error=True) as txn:
             if not self._nb_idl.get_port_group(pg_name):
                 # If drop Port Group doesn't exist yet, create it.
-                txn.add(self._nb_idl.pg_add(pg_name, acls=[], may_exist=True))
+                txn.add(self._nb_idl.pg_add(pg_name, acls=[]))
                 # Add ACLs to this Port Group so that all traffic is dropped.
                 acls = ovn_acl.add_acls_for_drop_port_group(pg_name)
                 for acl in acls:
-                    txn.add(self._nb_idl.pg_acl_add(may_exist=True, **acl))
+                    txn.add(self._nb_idl.pg_acl_add(**acl))
 
             if ports:
                 ports_ids = [port['id'] for port in ports]
@@ -1829,10 +1777,10 @@ class OVNClient(object):
         ports = self._plugin.get_ports(context, filters=dict(
             network_id=[network_id], device_owner=[const.DEVICE_OWNER_DHCP]))
 
-        # Metadata ports are DHCP ports not belonging to the Neutron
-        # DHCP agents
+        # Metadata ports are DHCP ports without a device_id starting by 'dhcp'
+        # since those belong to Neutron DHCP agents.
         for port in ports:
-            if not utils.is_neutron_dhcp_agent_port(port):
+            if not port['device_id'].startswith('dhcp'):
                 return port
 
     def _find_metadata_port_ip(self, context, subnet):
@@ -1842,17 +1790,10 @@ class OVNClient(object):
                 if fixed_ip['subnet_id'] == subnet['id']:
                     return fixed_ip['ip_address']
 
-    def _get_metadata_ports(self, context, network_id):
-        if not config.is_ovn_metadata_enabled():
-            return
-
-        return self._plugin.get_ports(context, filters=dict(
-            network_id=[network_id], device_owner=[const.DEVICE_OWNER_DHCP]))
-
     def create_metadata_port(self, context, network):
         if config.is_ovn_metadata_enabled():
-            metadata_ports = self._get_metadata_ports(context, network['id'])
-            if not metadata_ports:
+            metadata_port = self._find_metadata_port(context, network['id'])
+            if not metadata_port:
                 # Create a neutron port for DHCP/metadata services
                 port = {'port':
                         {'network_id': network['id'],
@@ -1861,10 +1802,6 @@ class OVNClient(object):
                          'device_id': 'ovnmeta-%s' % network['id']}}
                 # TODO(boden): rehome create_port into neutron-lib
                 p_utils.create_port(self._plugin, context, port)
-            elif len(metadata_ports) > 1:
-                LOG.error("More than one metadata ports found for network %s. "
-                          "Please run the neutron-ovn-db-sync-util to fix it.",
-                          network['id'])
 
     def update_metadata_port(self, context, network_id):
         """Update metadata port.
