@@ -27,7 +27,6 @@ from neutron_lib.services import base as service_base
 from oslo_log import log
 from oslo_utils import excutils
 
-from neutron.db import common_db_mixin
 from neutron.db import dns_db
 from neutron.db import extraroute_db
 from neutron.db import l3_gwmode_db
@@ -48,7 +47,6 @@ LOG = log.getLogger(__name__)
 
 @registry.has_registry_receivers
 class OVNL3RouterPlugin(service_base.ServicePluginBase,
-                        common_db_mixin.CommonDbMixin,
                         extraroute_db.ExtraRoute_dbonly_mixin,
                         l3_gwmode_db.L3_NAT_db_mixin,
                         dns_db.DNSDbMixin):
@@ -321,35 +319,10 @@ class OVNL3RouterPlugin(service_base.ServicePluginBase,
             port_physnet_dict, chassis_physnets, cms)
         for g_name in unhosted_gateways:
             physnet = port_physnet_dict.get(g_name[len('lrp-'):])
-            # Remove any invalid gateway chassis from the list, otherwise
-            # we can have a situation where all existing_chassis are invalid
-            existing_chassis = self._ovn.get_gateway_chassis_binding(g_name)
-            master = existing_chassis[0] if existing_chassis else None
-            existing_chassis = self.scheduler.filter_existing_chassis(
-                nb_idl=self._ovn, gw_chassis=cms,
-                physnet=physnet, chassis_physnets=chassis_physnets,
-                existing_chassis=existing_chassis)
             candidates = self._ovn_client.get_candidates_for_scheduling(
                 physnet, cms=cms, chassis_physnets=chassis_physnets)
             chassis = self.scheduler.select(
-                self._ovn, self._sb_ovn, g_name, candidates=candidates,
-                existing_chassis=existing_chassis)
-            if master and master != chassis[0]:
-                if master not in chassis:
-                    LOG.debug("Master gateway chassis %(old)s "
-                              "has been removed from the system. Moving "
-                              "gateway %(gw)s to other chassis %(new)s.",
-                              {'gw': g_name,
-                               'old': master,
-                               'new': chassis[0]})
-                else:
-                    LOG.debug("Gateway %s is hosted at %s.", g_name, master)
-                    # NOTE(mjozefcz): It means scheduler moved master chassis
-                    # to other gw based on scheduling method. But we don't
-                    # want network flap - so moving actual master to be on
-                    # the top.
-                    index = chassis.index(master)
-                    chassis[0], chassis[index] = chassis[index], chassis[0]
+                self._ovn, self._sb_ovn, g_name, candidates=candidates)
             # NOTE(dalvarez): Let's commit the changes in separate transactions
             # as we will rely on those for scheduling subsequent gateways.
             with self._ovn.transaction(check_error=True) as txn:
@@ -374,7 +347,7 @@ class OVNL3RouterPlugin(service_base.ServicePluginBase,
             'device_owner': [n_const.DEVICE_OWNER_ROUTER_GW],
             'fixed_ips': {'subnet_id': [orig['id']]},
         })
-        router_ids = set([port['device_id'] for port in gw_ports])
+        router_ids = {port['device_id'] for port in gw_ports}
         remove = [{'destination': '0.0.0.0/0', 'nexthop': orig_gw_ip}
                   ] if orig_gw_ip else []
         add = [{'destination': '0.0.0.0/0', 'nexthop': current_gw_ip}
