@@ -12,7 +12,6 @@
 #    under the License.
 #
 
-import copy
 import functools
 import operator
 import threading
@@ -53,7 +52,7 @@ from networking_ovn.ml2 import qos_driver
 from networking_ovn.ml2 import trunk_driver
 from networking_ovn import ovn_db_sync
 from networking_ovn.ovsdb import impl_idl_ovn
-from networking_ovn.ovsdb import ovsdb_monitor
+from networking_ovn.ovsdb import worker
 
 
 LOG = log.getLogger(__name__)
@@ -194,7 +193,7 @@ class OVNMechanismDriver(api.MechanismDriver):
         # Now IDL connections can be safely used.
         self._post_fork_event.set()
 
-        if utils.get_method_class(trigger) == ovsdb_monitor.OvnWorker:
+        if utils.get_method_class(trigger) == worker.OvnWorker:
             # Call the synchronization task if its ovn worker
             # This sync neutron DB to OVN-NB DB only in inconsistent states
             self.nb_synchronizer = ovn_db_sync.OvnNbSynchronizer(
@@ -273,7 +272,7 @@ class OVNMechanismDriver(api.MechanismDriver):
             LOG.debug('Validating network segment with '
                       'type %(network_type)s, '
                       'segmentation ID %(segmentation_id)s, '
-                      'physical network %(physical_network)s' %
+                      'physical network %(physical_network)s',
                       {'network_type': network_type,
                        'segmentation_id': segmentation_id,
                        'physical_network': physical_network})
@@ -485,7 +484,7 @@ class OVNMechanismDriver(api.MechanismDriver):
         drastically affect performance.  Raising an exception will
         result in the deletion of the resource.
         """
-        port = copy.deepcopy(context.current)
+        port = context.current
         port['network'] = context.network.current
         self._ovn_client.create_port(port)
         self._notify_dhcp_updated(port['id'])
@@ -539,30 +538,10 @@ class OVNMechanismDriver(api.MechanismDriver):
         state. It is up to the mechanism driver to ignore state or
         state changes that it does not know or care about.
         """
-        port = copy.deepcopy(context.current)
+        port = context.current
         port['network'] = context.network.current
-        original_port = copy.deepcopy(context.original)
+        original_port = context.original
         original_port['network'] = context.network.current
-
-        # NOTE(mjozefcz): Check if port is in migration state. If so update
-        # the port status from DOWN to UP in order to generate 'fake'
-        # vif-interface-plugged event. This workaround is needed to
-        # perform live-migration with live_migration_wait_for_vif_plug=True.
-        if ((port['status'] == const.PORT_STATUS_DOWN and
-             ovn_const.MIGRATING_ATTR in port[portbindings.PROFILE].keys() and
-             port[portbindings.VIF_TYPE] == portbindings.VIF_TYPE_OVS)):
-            admin_context = n_context.get_admin_context()
-            LOG.info("Setting port %s status from DOWN to UP in order "
-                     "to emit vif-interface-plugged event.",
-                     port['id'])
-            self._plugin.update_port_status(admin_context, port['id'],
-                                            const.PORT_STATUS_ACTIVE)
-            # The revision has been changed. In the meantime
-            # port-update event already updated the OVN configuration,
-            # So there is no need to update it again here. Anyway it
-            # will fail that OVN has port with bigger revision.
-            return
-
         self._ovn_client.update_port(port, port_object=original_port)
         self._notify_dhcp_updated(port['id'])
 
@@ -578,7 +557,7 @@ class OVNMechanismDriver(api.MechanismDriver):
         expected, and will not prevent the resource from being
         deleted.
         """
-        port = copy.deepcopy(context.current)
+        port = context.current
         port['network'] = context.network.current
         self._ovn_client.delete_port(port['id'], port_object=port)
 
@@ -627,7 +606,7 @@ class OVNMechanismDriver(api.MechanismDriver):
         vnic_type = port.get(portbindings.VNIC_TYPE, portbindings.VNIC_NORMAL)
         if vnic_type not in self.supported_vnic_types:
             LOG.debug('Refusing to bind port %(port_id)s due to unsupported '
-                      'vnic_type: %(vnic_type)s' %
+                      'vnic_type: %(vnic_type)s',
                       {'port_id': port['id'], 'vnic_type': vnic_type})
             return
 
@@ -637,7 +616,7 @@ class OVNMechanismDriver(api.MechanismDriver):
             capabilities = profile.get('capabilities', [])
         if (vnic_type == portbindings.VNIC_DIRECT and
            'switchdev' not in capabilities):
-            LOG.debug("Refusing to bind port due to unsupported vnic_type: %s"
+            LOG.debug("Refusing to bind port due to unsupported vnic_type: %s "
                       "with no switchdev capability", portbindings.VNIC_DIRECT)
             return
 
@@ -651,7 +630,7 @@ class OVNMechanismDriver(api.MechanismDriver):
             iface_types = iface_types.split(',') if iface_types else []
         except RuntimeError:
             LOG.debug('Refusing to bind port %(port_id)s due to '
-                      'no OVN chassis for host: %(host)s' %
+                      'no OVN chassis for host: %(host)s',
                       {'port_id': port['id'], 'host': context.host})
             return
 
@@ -662,7 +641,7 @@ class OVNMechanismDriver(api.MechanismDriver):
             LOG.debug('Attempting to bind port %(port_id)s on host %(host)s '
                       'for network segment with type %(network_type)s, '
                       'segmentation ID %(segmentation_id)s, '
-                      'physical network %(physical_network)s' %
+                      'physical network %(physical_network)s',
                       {'port_id': port['id'],
                        'host': context.host,
                        'network_type': network_type,
@@ -717,7 +696,20 @@ class OVNMechanismDriver(api.MechanismDriver):
         workers, can return a sequence of worker instances.
         """
         # See doc/source/design/ovn_worker.rst for more details.
-        return [ovsdb_monitor.OvnWorker(), maintenance.MaintenanceWorker()]
+        return [worker.OvnWorker(), maintenance.MaintenanceWorker()]
+
+    def _update_subport_host_if_needed(self, port_id):
+        parent_port = self._ovn_client.get_parent_port(port_id)
+        if parent_port:
+            admin_context = n_context.get_admin_context()
+            try:
+                port = self._plugin.get_port(admin_context, parent_port)
+                host_id = port.get(portbindings.HOST_ID, '')
+                subport = {'port': {'binding:host_id': host_id}}
+                self._plugin.update_port(admin_context, port_id, subport)
+            except (os_db_exc.DBReferenceError, n_exc.PortNotFound):
+                LOG.debug("Error trying to set host_id %s for subport %s",
+                          host_id, port_id)
 
     def _update_dnat_entry_if_needed(self, port_id, up=True):
         """Update DNAT entry if using distributed floating ips."""
@@ -764,6 +756,11 @@ class OVNMechanismDriver(api.MechanismDriver):
         self._update_dnat_entry_if_needed(port_id)
         self._wait_for_metadata_provisioned_if_needed(port_id)
 
+        # If this port is a subport, we need to update the host_id and set it
+        # to its parent's. Otherwise, Neutron won't even try to bind it and
+        # it will not transition from DOWN to ACTIVE.
+        self._update_subport_host_if_needed(port_id)
+
         provisioning_blocks.provisioning_complete(
             n_context.get_admin_context(),
             port_id,
@@ -861,8 +858,8 @@ class OVNMechanismDriver(api.MechanismDriver):
         """Wait for metadata service to be provisioned.
 
         Wait until metadata service has been setup for this port in the chassis
-        it resides. If metadata is disabled or DHCP is not enabled for its
-        subnets, this function will return right away.
+        it resides. If metadata is disabled, this function will return right
+        away.
         """
         if config.is_ovn_metadata_enabled() and self._sb_ovn:
             # Wait until metadata service has been setup for this port in the
@@ -877,26 +874,6 @@ class OVNMechanismDriver(api.MechanismDriver):
                 LOG.warning("Logical port %s is not bound to a "
                             "chassis", port_id)
                 return
-
-            # Check if the port belongs to some IPv4 subnet with DHCP enabled.
-            context = n_context.get_admin_context()
-            port = self._plugin.get_port(context, port_id)
-            port_subnet_ids = set(
-                ip['subnet_id'] for ip in port['fixed_ips'] if
-                n_utils.get_ip_version(ip['ip_address']) == const.IP_VERSION_4)
-            if not port_subnet_ids:
-                # The port doesn't belong to any IPv4 subnet
-                return
-
-            subnets = self._plugin.get_subnets(context, filters=dict(
-                network_id=[port['network_id']], ip_version=[4],
-                enable_dhcp=True))
-
-            subnet_ids = set(
-                s['id'] for s in subnets if s['id'] in port_subnet_ids)
-            if not subnet_ids:
-                return
-
             try:
                 n_utils.wait_until_true(
                     lambda: datapath in
