@@ -407,7 +407,6 @@ class DelLRouterPortCommand(command.BaseCommand):
             raise RuntimeError(msg)
 
         _delvalue_from_list(lrouter, 'ports', lrouter_port)
-        lrouter_port.delete()
 
 
 class SetLRouterPortInLSwitchPortCommand(command.BaseCommand):
@@ -972,71 +971,6 @@ class SetNATRuleInLRouterCommand(command.BaseCommand):
                 break
 
 
-class AddNatIpToLRPortPeerOptionsCommand(command.BaseCommand):
-    # TODO(chandrav): Add unit tests, bug #1638715.
-    def __init__(self, api, lport, nat_ip):
-        super(AddNatIpToLRPortPeerOptionsCommand, self).__init__(api)
-        self.lport = lport
-        self.nat_ip = nat_ip
-
-    def run_idl(self, txn):
-        try:
-            lport = idlutils.row_by_value(self.api.idl, 'Logical_Switch_Port',
-                                          'name', self.lport)
-        except idlutils.RowNotFound:
-            msg = _("Logical Switch Port %s does not exist") % self.lport
-            raise RuntimeError(msg)
-
-        lport.verify('addresses')
-        addresses = getattr(lport, 'addresses', [])
-        lport.verify('options')
-        options = getattr(lport, 'options', {})
-
-        nat_ip_list = options.get('nat-addresses', '').split()
-        if not nat_ip_list:
-            nat_ips = addresses[0].split()[0] + ' ' + self.nat_ip
-        elif self.nat_ip not in nat_ip_list:
-            nat_ip_list.append(self.nat_ip)
-            nat_ips = ' '.join(nat_ip_list)
-        else:
-            return
-        options['nat-addresses'] = nat_ips
-        lport.options = options
-
-
-class DeleteNatIpFromLRPortPeerOptionsCommand(command.BaseCommand):
-    # TODO(chandrav): Add unit tests, bug #1638715.
-    def __init__(self, api, lport, nat_ip):
-        super(DeleteNatIpFromLRPortPeerOptionsCommand, self).__init__(api)
-        self.lport = lport
-        self.nat_ip = nat_ip
-
-    def run_idl(self, txn):
-        try:
-            lport = idlutils.row_by_value(self.api.idl, 'Logical_Switch_Port',
-                                          'name', self.lport)
-
-        except idlutils.RowNotFound:
-            msg = _("Logical Switch Port %s does not exist") % self.lport
-            raise RuntimeError(msg)
-
-        lport.verify('options')
-        options = getattr(lport, 'options', {})
-        nat_ip_list = options.get('nat-addresses', '').split()
-        if self.nat_ip not in nat_ip_list:
-            return
-        nat_ip_list.remove(self.nat_ip)
-        if len(nat_ip_list) is 1:
-            nat_ips = ''
-        else:
-            nat_ips = ' '.join(nat_ip_list)
-        if not nat_ips:
-            del options['nat-addresses']
-        else:
-            options['nat-addresses'] = nat_ips
-        lport.options = options
-
-
 class CheckRevisionNumberCommand(command.BaseCommand):
 
     def __init__(self, api, name, resource, resource_type, if_exists):
