@@ -14,11 +14,9 @@
 
 import abc
 
-from neutron.common import config
 from neutron_lib.plugins import constants
 from neutron_lib.plugins import directory
 from neutron_lib.utils import helpers
-from neutron_lib import worker
 from oslo_log import log
 from ovs.stream import Stream
 from ovsdbapp.backend.ovs_idl import connection
@@ -100,6 +98,10 @@ class ChassisMetadataAgentEvent(BaseEvent):
             stats.AgentStats.add_stat(utils.ovn_metadata_name(row.uuid),
                                       self._metadata_nb_cfg(row))
         except (AttributeError, KeyError):
+            LOG.warning('No "%(key)s" key found for the metadata agent at '
+                        'Chassis %(chassis)s',
+                        {'key': ovn_const.OVN_AGENT_METADATA_SB_CFG_KEY,
+                         'chassis': row.uuid})
             return
 
 
@@ -128,46 +130,6 @@ class ChassisEvent(row_event.RowEvent):
             self.l3_plugin.schedule_unhosted_gateways()
 
 
-class PortBindingChassisUpdateEvent(BaseEvent):
-    """Event for matching a port moving chassis
-
-    If the LSP is up and the Port_Binding chassis has just changed,
-    there is a good chance the host died without cleaning up the chassis
-    column on the Port_Binding. The port never goes down, so we won't
-    see update the driver with the LogicalSwitchPortUpdateUpEvent which
-    only monitors for transitions from DOWN to UP.
-    """
-
-    table = 'Port_Binding'
-    events = (BaseEvent.ROW_UPDATE,)
-
-    def __init__(self, driver):
-        self.driver = driver
-        super(PortBindingChassisUpdateEvent, self).__init__()
-
-    def match_fn(self, event, row, old=None):
-        # NOTE(twilson) ROW_UPDATE events always pass old, but chassis will
-        # only be set if chassis has changed
-        old_chassis = getattr(old, 'chassis', None)
-        if not (row.chassis and old_chassis) or row.chassis == old_chassis:
-            return False
-        if row.type == 'chassisredirect':
-            return False
-        try:
-            lsp = self.driver._nb_ovn.lookup('Logical_Switch_Port',
-                                             row.logical_port)
-        except idlutils.RowNotFound:
-            LOG.warning("Logical Switch Port %(port)s not found for "
-                        "Port_Binding %(binding)s",
-                        {'port': row.logical_port, 'binding': row.uuid})
-            return False
-
-        return bool(lsp.up)
-
-    def run(self, event, row, old=None):
-        self.driver.set_port_status_up(row.logical_port)
-
-
 class PortBindingChassisEvent(row_event.RowEvent):
     """Port_Binding update event - set chassis for chassisredirect port.
 
@@ -181,7 +143,7 @@ class PortBindingChassisEvent(row_event.RowEvent):
         self.driver = driver
         self.l3_plugin = directory.get_plugin(constants.L3)
         table = 'Port_Binding'
-        events = (self.ROW_UPDATE)
+        events = (self.ROW_UPDATE,)
         super(PortBindingChassisEvent, self).__init__(
             events, table, (('type', '=', 'chassisredirect'),))
         self.event_name = 'PortBindingChassisEvent'
@@ -212,7 +174,7 @@ class LogicalSwitchPortCreateUpEvent(row_event.RowEvent):
     def __init__(self, driver):
         self.driver = driver
         table = 'Logical_Switch_Port'
-        events = (self.ROW_CREATE)
+        events = (self.ROW_CREATE,)
         super(LogicalSwitchPortCreateUpEvent, self).__init__(
             events, table, (('up', '=', True),))
         self.event_name = 'LogicalSwitchPortCreateUpEvent'
@@ -231,7 +193,7 @@ class LogicalSwitchPortCreateDownEvent(row_event.RowEvent):
     def __init__(self, driver):
         self.driver = driver
         table = 'Logical_Switch_Port'
-        events = (self.ROW_CREATE)
+        events = (self.ROW_CREATE,)
         super(LogicalSwitchPortCreateDownEvent, self).__init__(
             events, table, (('up', '=', False),))
         self.event_name = 'LogicalSwitchPortCreateDownEvent'
@@ -250,7 +212,7 @@ class LogicalSwitchPortUpdateUpEvent(row_event.RowEvent):
     def __init__(self, driver):
         self.driver = driver
         table = 'Logical_Switch_Port'
-        events = (self.ROW_UPDATE)
+        events = (self.ROW_UPDATE,)
         super(LogicalSwitchPortUpdateUpEvent, self).__init__(
             events, table, (('up', '=', True),),
             old_conditions=(('up', '=', False),))
@@ -270,7 +232,7 @@ class LogicalSwitchPortUpdateDownEvent(row_event.RowEvent):
     def __init__(self, driver):
         self.driver = driver
         table = 'Logical_Switch_Port'
-        events = (self.ROW_UPDATE)
+        events = (self.ROW_UPDATE,)
         super(LogicalSwitchPortUpdateDownEvent, self).__init__(
             events, table, (('up', '=', False),),
             old_conditions=(('up', '=', True),))
@@ -320,9 +282,10 @@ class BaseOvnSbIdl(connection.OvsdbIdl):
     def __init__(self, remote, schema):
         super(BaseOvnSbIdl, self).__init__(remote, schema)
         self.notify_handler = event.RowEventHandler()
-        self.notify_handler.watch_events([
-            ChassisAgentDeleteEvent(), ChassisMetadataAgentEvent(),
-            ChassisGatewayAgentEvent()])
+        events = [ChassisAgentDeleteEvent(), ChassisGatewayAgentEvent()]
+        if ovn_config.is_ovn_metadata_enabled():
+            events.append(ChassisMetadataAgentEvent())
+        self.notify_handler.watch_events(events)
 
     @classmethod
     def from_server(cls, connection_string, schema_name):
@@ -370,9 +333,9 @@ class OvnIdl(BaseOvnIdl):
             return
         self.notify_handler.notify(event, row, updates)
 
+    @abc.abstractmethod
     def post_connect(self):
         """Should be called after the idl has been initialized"""
-        pass
 
 
 class OvnNbIdl(OvnIdl):
@@ -444,9 +407,8 @@ class OvnSbIdl(OvnIdl):
         """
         self._chassis_event = ChassisEvent(self.driver)
         self._portbinding_event = PortBindingChassisEvent(self.driver)
-        self.notify_handler.watch_events(
-            [self._chassis_event, self._portbinding_event,
-             PortBindingChassisUpdateEvent(self.driver)])
+        self.notify_handler.watch_events([self._chassis_event,
+                                          self._portbinding_event])
 
 
 def _check_and_set_ssl_files(schema_name):
@@ -467,23 +429,3 @@ def _check_and_set_ssl_files(schema_name):
 
     if ca_cert_file:
         Stream.ssl_set_ca_cert_file(ca_cert_file)
-
-
-class OvnWorker(worker.BaseWorker):
-    def start(self):
-        super(OvnWorker, self).start()
-        # NOTE(twilson) The super class will trigger the post_fork_initialize
-        # in the driver, which starts the connection/IDL notify loop which
-        # keeps the process from exiting
-
-    def stop(self):
-        """Stop service."""
-        # TODO(numans)
-
-    def wait(self):
-        """Wait for service to complete."""
-        # TODO(numans)
-
-    @staticmethod
-    def reset():
-        config.reset_service()
