@@ -37,6 +37,7 @@ from ovsdbapp.backend.ovs_idl import connection
 from networking_ovn.db import models  # noqa
 from networking_ovn.ovsdb import impl_idl_ovn
 from networking_ovn.ovsdb import ovsdb_monitor
+from networking_ovn.ovsdb import worker
 from networking_ovn.tests import base
 from networking_ovn.tests.functional.resources import process
 
@@ -247,7 +248,7 @@ class TestOVNFunctionalBase(test_plugin.Ml2PluginV2TestCase):
 
         trigger_cls = TriggerCls()
         if self.ovn_worker:
-            trigger_cls.trigger.__self__.__class__ = ovsdb_monitor.OvnWorker
+            trigger_cls.trigger.__self__.__class__ = worker.OvnWorker
             cfg.CONF.set_override('neutron_sync_mode', 'off', 'ovn')
 
         self.addCleanup(self.stop)
@@ -287,15 +288,13 @@ class TestOVNFunctionalBase(test_plugin.Ml2PluginV2TestCase):
         self._start_ovsdb_server_and_idls()
         self._start_ovn_northd()
 
-    def add_fake_chassis(self, host, physical_nets=None, external_ids=None,
-                         name=None):
+    def add_fake_chassis(self, host, physical_nets=None, external_ids=None):
         physical_nets = physical_nets or []
         external_ids = external_ids or {}
 
         bridge_mapping = ",".join(["%s:br-provider%s" % (phys_net, i)
                                   for i, phys_net in enumerate(physical_nets)])
-        if name is None:
-            name = uuidutils.generate_uuid()
+        name = uuidutils.generate_uuid()
         external_ids['ovn-bridge-mappings'] = bridge_mapping
         # We'll be using different IP addresses every time for the Encap of
         # the fake chassis as the SB schema doesn't allow to have two entries
@@ -309,7 +308,3 @@ class TestOVNFunctionalBase(test_plugin.Ml2PluginV2TestCase):
             name, ['geneve'], '172.24.4.%d' % self._counter,
             external_ids=external_ids, hostname=host).execute(check_error=True)
         return name
-
-    def del_fake_chassis(self, chassis, if_exists=True):
-        self.sb_api.chassis_del(
-            chassis, if_exists=if_exists).execute(check_error=True)
