@@ -69,9 +69,7 @@ class OVNL3RouterPlugin(test_mech_driver.OVNMechanismDriverTestCase):
             'networks': ['10.0.0.100/24'],
             'external_ids': {
                 ovn_const.OVN_SUBNET_EXT_IDS_KEY: 'subnet-id',
-                ovn_const.OVN_REV_NUM_EXT_ID_KEY: '1',
-                ovn_const.OVN_NETWORK_NAME_EXT_ID_KEY:
-                utils.ovn_name(self.fake_network['id'])}}
+                ovn_const.OVN_REV_NUM_EXT_ID_KEY: '1'}}
         self.fake_router_ports = [self.fake_router_port]
         self.fake_subnet = {'id': 'subnet-id',
                             'ip_version': 4,
@@ -125,9 +123,7 @@ class OVNL3RouterPlugin(test_mech_driver.OVNMechanismDriverTestCase):
             'may_exist': True,
             'external_ids': {
                 ovn_const.OVN_SUBNET_EXT_IDS_KEY: 'ext-subnet-id',
-                ovn_const.OVN_REV_NUM_EXT_ID_KEY: '1',
-                ovn_const.OVN_NETWORK_NAME_EXT_ID_KEY:
-                utils.ovn_name(self.fake_network['id'])},
+                ovn_const.OVN_REV_NUM_EXT_ID_KEY: '1'},
             'gateway_chassis': ['hv1']}
         self.fake_floating_ip_attrs = {'floating_ip_address': '192.168.0.10',
                                        'fixed_ip_address': '10.0.0.10'}
@@ -151,11 +147,11 @@ class OVNL3RouterPlugin(test_mech_driver.OVNMechanismDriverTestCase):
                 ovn_const.OVN_ROUTER_NAME_EXT_ID_KEY: utils.ovn_name(
                     self.fake_floating_ip['router_id'])}}
         self.l3_inst = directory.get_plugin(plugin_constants.L3)
-        self.nb_idl = self._start_mock(
+        self._start_mock(
             'networking_ovn.l3.l3_ovn.OVNL3RouterPlugin._ovn',
             new_callable=mock.PropertyMock,
             return_value=fakes.FakeOvsdbNbOvnIdl())
-        self.sb_idl = self._start_mock(
+        self._start_mock(
             'networking_ovn.l3.l3_ovn.OVNL3RouterPlugin._sb_ovn',
             new_callable=mock.PropertyMock,
             return_value=fakes.FakeOvsdbSbOvnIdl())
@@ -183,11 +179,11 @@ class OVNL3RouterPlugin(test_mech_driver.OVNMechanismDriverTestCase):
         self._start_mock(
             'neutron.db.l3_db.L3_NAT_dbonly_mixin.delete_router',
             return_value={})
-        self.mock_candidates = self._start_mock(
+        self._start_mock(
             'networking_ovn.common.ovn_client.'
             'OVNClient.get_candidates_for_scheduling',
             return_value=[])
-        self.mock_schedule = self._start_mock(
+        self._start_mock(
             'networking_ovn.l3.l3_ovn_scheduler.'
             'OVNGatewayLeastLoadedScheduler._schedule_gateway',
             return_value=['hv1'])
@@ -250,8 +246,7 @@ class OVNL3RouterPlugin(test_mech_driver.OVNMechanismDriverTestCase):
             'fixed_ips': [
                 {'ip_address': '2001:db8::1', 'subnet_id': 'subnet-id1'},
                 {'ip_address': '2001:dba::1', 'subnet_id': 'subnet-id2'}],
-            'mac_address': 'aa:aa:aa:aa:aa:aa',
-            'network_id': 'network-id1'
+            'mac_address': 'aa:aa:aa:aa:aa:aa'
             }
         fake_rtr_intf_networks = ['2001:db8::1/24', '2001:dba::1/24']
         self.l3_inst.add_router_interface(self.context, router_id,
@@ -293,9 +288,7 @@ class OVNL3RouterPlugin(test_mech_driver.OVNMechanismDriverTestCase):
             networks=['10.0.0.100/24'],
             external_ids={
                 ovn_const.OVN_SUBNET_EXT_IDS_KEY: 'subnet-id',
-                ovn_const.OVN_REV_NUM_EXT_ID_KEY: '1',
-                ovn_const.OVN_NETWORK_NAME_EXT_ID_KEY:
-                utils.ovn_name(self.fake_network['id'])})
+                ovn_const.OVN_REV_NUM_EXT_ID_KEY: '1'})
 
     @mock.patch('neutron.db.extraroute_db.ExtraRoute_dbonly_mixin.'
                 'update_router')
@@ -500,6 +493,48 @@ class OVNL3RouterPlugin(test_mech_driver.OVNMechanismDriverTestCase):
                 lsp_address=ovn_const.DEFAULT_ADDR_FOR_LSP_WITH_PEER)
         self.l3_inst._ovn.add_nat_rule_in_lrouter.assert_not_called()
 
+    @mock.patch('neutron.db.db_base_plugin_v2.NeutronDbPluginV2.get_network')
+    @mock.patch('neutron.db.db_base_plugin_v2.NeutronDbPluginV2.get_port')
+    @mock.patch('neutron.db.db_base_plugin_v2.NeutronDbPluginV2.get_subnet')
+    @mock.patch('networking_ovn.common.ovn_client.OVNClient._get_router_ports')
+    @mock.patch('neutron.db.l3_db.L3_NAT_dbonly_mixin.get_router')
+    @mock.patch('neutron.db.l3_db.L3_NAT_dbonly_mixin.add_router_interface')
+    def test_add_router_interface_vlan_network(self, ari, gr, grps, gs,
+                                               gp, gn):
+        router_id = 'router-id'
+        interface_info = {'port_id': 'router-port-id'}
+        ari.return_value = self.fake_router_interface_info
+        gr.return_value = self.fake_router_with_ext_gw
+        gs.return_value = self.fake_subnet
+        gp.return_value = self.fake_router_port
+
+        # Set the type to be VLAN
+        fake_network_vlan = self.fake_network
+        fake_network_vlan[pnet.NETWORK_TYPE] = constants.TYPE_VLAN
+        gn.return_value = fake_network_vlan
+
+        self.l3_inst.add_router_interface(self.context, router_id,
+                                          interface_info)
+
+        # Make sure that the "reside-on-redirect-chassis" option was
+        # set to the new router port
+        fake_router_port_assert = self.fake_router_port_assert
+        fake_router_port_assert['options'] = {
+            'reside-on-redirect-chassis': 'true'}
+
+        self.l3_inst._ovn.add_lrouter_port.assert_called_once_with(
+            **fake_router_port_assert)
+        self.l3_inst._ovn.set_lrouter_port_in_lswitch_port.\
+            assert_called_once_with(
+                'router-port-id', 'lrp-router-port-id', is_gw_port=False,
+                lsp_address=ovn_const.DEFAULT_ADDR_FOR_LSP_WITH_PEER)
+        self.l3_inst._ovn.add_nat_rule_in_lrouter.assert_called_once_with(
+            'neutron-router-id', logical_ip='10.0.0.0/24',
+            external_ip='192.168.1.1', type='snat')
+
+        self.bump_rev_p.assert_called_with(self.fake_router_port,
+                                           ovn_const.TYPE_ROUTER_PORTS)
+
     @mock.patch('neutron.db.db_base_plugin_v2.NeutronDbPluginV2.get_port')
     @mock.patch('neutron.db.db_base_plugin_v2.NeutronDbPluginV2.get_subnet')
     @mock.patch('neutron.db.l3_db.L3_NAT_dbonly_mixin.get_router')
@@ -801,8 +836,7 @@ class OVNL3RouterPlugin(test_mech_driver.OVNMechanismDriverTestCase):
     @mock.patch('neutron.db.l3_db.L3_NAT_dbonly_mixin._get_floatingip')
     def test_create_floatingip_distributed(self, gf, gp):
         self.l3_inst._ovn.is_col_present.return_value = True
-        gp.return_value = {'mac_address': '00:01:02:03:04:05',
-                           'network_id': 'port-network-id'}
+        gp.return_value = {'mac_address': '00:01:02:03:04:05'}
         gf.return_value = {'floating_port_id': 'fip-port-id'}
         config.cfg.CONF.set_override(
             'enable_distributed_floating_ip', True, group='ovn')
@@ -857,6 +891,7 @@ class OVNL3RouterPlugin(test_mech_driver.OVNMechanismDriverTestCase):
             {'external_ip': '192.168.0.10', 'logical_ip': '10.0.0.6',
              'type': 'dnat_and_snat', 'uuid': 'uuid1'}]
         self.l3_inst.create_floatingip(self.context, 'floatingip')
+        self.l3_inst._ovn.add_nat_rule_in_lrouter.assert_not_called()
         expected_ext_ids = {
             ovn_const.OVN_FIP_EXT_ID_KEY: self.fake_floating_ip['id'],
             ovn_const.OVN_REV_NUM_EXT_ID_KEY: '1',
@@ -864,8 +899,8 @@ class OVNL3RouterPlugin(test_mech_driver.OVNMechanismDriverTestCase):
                 self.fake_floating_ip['port_id'],
             ovn_const.OVN_ROUTER_NAME_EXT_ID_KEY: utils.ovn_name(
                 self.fake_floating_ip['router_id'])}
-        self.l3_inst._ovn.add_nat_rule_in_lrouter.assert_called_once_with(
-            'neutron-router-id',
+        self.l3_inst._ovn.set_nat_rule_in_lrouter.assert_called_once_with(
+            'neutron-router-id', 'uuid1',
             type='dnat_and_snat',
             logical_ip='10.0.0.10',
             external_ip='192.168.0.10',
@@ -898,19 +933,6 @@ class OVNL3RouterPlugin(test_mech_driver.OVNMechanismDriverTestCase):
         self.l3_inst._ovn.delete_lswitch_port.assert_called_once_with(
             'fip-port-id', 'neutron-fip-net-id')
 
-    @mock.patch('neutron.db.l3_db.L3_NAT_dbonly_mixin._get_floatingip')
-    def test_create_floatingip_lsp_external_id(self, gf):
-        foo_lport = fakes.FakeOvsdbRow.create_one_ovsdb_row()
-        foo_lport.uuid = 'foo-port'
-        self.l3_inst._ovn.get_lswitch_port.return_value = foo_lport
-        self.l3_inst.create_floatingip(self.context, 'floatingip')
-        calls = [mock.call(
-            'Logical_Switch_Port',
-            'foo-port',
-            ('external_ids', {ovn_const.OVN_PORT_FIP_EXT_ID_KEY:
-                              '192.168.0.10'}))]
-        self.l3_inst._ovn.db_set.assert_has_calls(calls)
-
     @mock.patch('neutron.db.l3_db.L3_NAT_dbonly_mixin.delete_floatingip')
     def test_delete_floatingip(self, df):
         self.l3_inst._ovn.get_floatingip.return_value = (
@@ -922,35 +944,6 @@ class OVNL3RouterPlugin(test_mech_driver.OVNMechanismDriverTestCase):
             logical_ip='10.0.0.10',
             external_ip='192.168.0.10')
 
-    @mock.patch('neutron.db.l3_db.L3_NAT_dbonly_mixin._get_floatingip')
-    @mock.patch('neutron.db.l3_db.L3_NAT_dbonly_mixin.delete_floatingip')
-    def test_delete_floatingip_lsp_external_id(self, df, gf):
-        gf.return_value = self.fake_floating_ip
-        self.l3_inst._ovn.get_floatingip.return_value = (
-            self.fake_ovn_nat_rule)
-
-        foo_lport = fakes.FakeOvsdbRow.create_one_ovsdb_row()
-        foo_lport.uuid = 'foo-port'
-        foo_lport.external_ids = {
-            ovn_const.OVN_PORT_FIP_EXT_ID_KEY: 'foo-port'}
-        self.l3_inst._ovn.get_lswitch_port.return_value = foo_lport
-
-        self.l3_inst.delete_floatingip(self.context, 'floatingip-id')
-        calls = [mock.call(
-            'Logical_Switch_Port', 'foo-port',
-            'external_ids', ovn_const.OVN_PORT_FIP_EXT_ID_KEY)]
-        self.l3_inst._ovn.db_remove.assert_has_calls(calls)
-
-    @mock.patch('neutron.db.l3_db.L3_NAT_dbonly_mixin._get_floatingip')
-    @mock.patch('neutron.db.l3_db.L3_NAT_dbonly_mixin.delete_floatingip')
-    def test_delete_floatingip_no_lsp_external_id(self, df, gf):
-        gf.return_value = self.fake_floating_ip
-        self.l3_inst._ovn.get_floatingip.return_value = (
-            self.fake_ovn_nat_rule)
-        self.l3_inst._ovn.get_lswitch_port.return_value = None
-        self.l3_inst.delete_floatingip(self.context, 'floatingip-id')
-        self.l3_inst._ovn.db_remove.assert_not_called()
-
     @mock.patch('neutron.db.l3_db.L3_NAT_dbonly_mixin._get_floatingip')
     @mock.patch('neutron.db.extraroute_db.ExtraRoute_dbonly_mixin.'
                 'update_floatingip')
@@ -1004,24 +997,16 @@ class OVNL3RouterPlugin(test_mech_driver.OVNMechanismDriverTestCase):
             external_ip='192.168.0.10',
             external_ids=expected_ext_ids)
 
-    @mock.patch('neutron.db.db_base_plugin_v2.NeutronDbPluginV2.get_network')
     @mock.patch('neutron.db.db_base_plugin_v2.NeutronDbPluginV2.get_port')
     @mock.patch('neutron.db.l3_db.L3_NAT_dbonly_mixin._get_floatingip')
     @mock.patch('neutron.db.extraroute_db.ExtraRoute_dbonly_mixin.'
                 'update_floatingip')
-    def _test_update_floatingip_associate_distributed(self, network_type,
-                                                      uf, gf, gp, gn):
+    def test_update_floatingip_associate_distributed(self, uf, gf, gp):
         self.l3_inst._ovn.is_col_present.return_value = True
         self.fake_floating_ip.update({'fixed_port_id': None})
-        gp.return_value = {'mac_address': '00:01:02:03:04:05',
-                           'network_id': 'port-network-id'}
+        gp.return_value = {'mac_address': '00:01:02:03:04:05'}
         gf.return_value = self.fake_floating_ip
         uf.return_value = self.fake_floating_ip_new
-
-        fake_network_vlan = self.fake_network
-        fake_network_vlan[pnet.NETWORK_TYPE] = network_type
-        gn.return_value = fake_network_vlan
-
         config.cfg.CONF.set_override(
             'enable_distributed_floating_ip', True, group='ovn')
         self.l3_inst.update_floatingip(self.context, 'id', 'floatingip')
@@ -1034,54 +1019,23 @@ class OVNL3RouterPlugin(test_mech_driver.OVNMechanismDriverTestCase):
             ovn_const.OVN_ROUTER_NAME_EXT_ID_KEY: utils.ovn_name(
                 self.fake_floating_ip_new['router_id']),
             ovn_const.OVN_FIP_EXT_MAC_KEY: '00:01:02:03:04:05'}
-        if network_type == constants.TYPE_VLAN:
-            self.l3_inst._ovn.add_nat_rule_in_lrouter.assert_called_once_with(
-                'neutron-new-router-id', type='dnat_and_snat',
-                logical_ip='10.10.10.10', external_ip='192.168.0.10',
-                logical_port='new-port_id', external_ids=expected_ext_ids)
-        else:
-            self.l3_inst._ovn.add_nat_rule_in_lrouter.assert_called_once_with(
-                'neutron-new-router-id', type='dnat_and_snat',
-                logical_ip='10.10.10.10', external_ip='192.168.0.10',
-                external_mac='00:01:02:03:04:05', logical_port='new-port_id',
-                external_ids=expected_ext_ids)
-
-    def test_update_floatingip_associate_distributed_flat(self):
-        self._test_update_floatingip_associate_distributed(constants.TYPE_FLAT)
-
-    def test_update_floatingip_associate_distributed_vlan(self):
-        self._test_update_floatingip_associate_distributed(constants.TYPE_VLAN)
+        self.l3_inst._ovn.add_nat_rule_in_lrouter.assert_called_once_with(
+            'neutron-new-router-id', type='dnat_and_snat',
+            logical_ip='10.10.10.10', external_ip='192.168.0.10',
+            external_mac='00:01:02:03:04:05', logical_port='new-port_id',
+            external_ids=expected_ext_ids)
 
     @mock.patch('neutron.db.l3_db.L3_NAT_dbonly_mixin._get_floatingip')
     @mock.patch('neutron.db.extraroute_db.ExtraRoute_dbonly_mixin.'
                 'update_floatingip')
-    def test_update_floatingip_association_empty_update(self, uf, gf):
-        self.l3_inst._ovn.is_col_present.return_value = True
-        self.l3_inst._ovn.get_floatingip.return_value = (
-            self.fake_ovn_nat_rule)
-        self.fake_floating_ip.update({'fixed_port_id': 'foo'})
-        self.fake_floating_ip_new.update({'port_id': 'foo'})
+    def test_update_floatingip_association_not_changed(self, uf, gf):
+        self.fake_floating_ip.update({'fixed_port_id': None})
+        self.fake_floating_ip_new.update({'port_id': None})
         gf.return_value = self.fake_floating_ip
         uf.return_value = self.fake_floating_ip_new
         self.l3_inst.update_floatingip(self.context, 'id', 'floatingip')
-        self.l3_inst._ovn.delete_nat_rule_in_lrouter.assert_called_once_with(
-            'neutron-router-id',
-            type='dnat_and_snat',
-            logical_ip='10.0.0.10',
-            external_ip='192.168.0.10')
-        expected_ext_ids = {
-            ovn_const.OVN_FIP_EXT_ID_KEY: self.fake_floating_ip_new['id'],
-            ovn_const.OVN_REV_NUM_EXT_ID_KEY: '1',
-            ovn_const.OVN_FIP_PORT_EXT_ID_KEY:
-                self.fake_floating_ip_new['port_id'],
-            ovn_const.OVN_ROUTER_NAME_EXT_ID_KEY: utils.ovn_name(
-                self.fake_floating_ip_new['router_id'])}
-        self.l3_inst._ovn.add_nat_rule_in_lrouter.assert_called_once_with(
-            'neutron-new-router-id',
-            type='dnat_and_snat',
-            logical_ip='10.10.10.10',
-            external_ip='192.168.0.10',
-            external_ids=expected_ext_ids)
+        self.l3_inst._ovn.delete_nat_rule_in_lrouter.assert_not_called()
+        self.l3_inst._ovn.add_nat_rule_in_lrouter.assert_not_called()
 
     @mock.patch('neutron.db.l3_db.L3_NAT_dbonly_mixin._get_floatingip')
     @mock.patch('neutron.db.extraroute_db.ExtraRoute_dbonly_mixin.'
@@ -1195,62 +1149,6 @@ class OVNL3RouterPlugin(test_mech_driver.OVNMechanismDriverTestCase):
         mock_updt_status.assert_called_once_with(
             mock.ANY, fake_port_id, constants.PORT_STATUS_DOWN)
 
-    def test_schedule_unhosted_gateways_no_gateways(self):
-        self.nb_idl().get_unhosted_gateways.return_value = []
-        self.l3_inst.schedule_unhosted_gateways()
-        self.nb_idl().update_lrouter_port.assert_not_called()
-
-    def test_schedule_unhosted_gateways(self):
-        unhosted_gws = ['lrp-foo-1', 'lrp-foo-2', 'lrp-foo-3']
-        chassis_mappings = {
-            'chassis1': ['physnet1'],
-            'chassis2': ['physnet1'],
-            'chassis3': ['physnet1']}
-        chassis = ['chassis1', 'chassis2', 'chassis3']
-        self.sb_idl().get_chassis_and_physnets.return_value = (
-            chassis_mappings)
-        self.sb_idl().get_gateway_chassis_from_cms_options.return_value = (
-            chassis)
-        self.nb_idl().get_unhosted_gateways.return_value = unhosted_gws
-        # 1. port has 2 gateway chassis
-        # 2. port has only chassis2
-        # 3. port is not bound
-        existing_port_bindings = [
-            ['chassis1', 'chassis2'],
-            ['chassis2'],
-            []]
-        self.nb_idl().get_gateway_chassis_binding.side_effect = (
-            existing_port_bindings)
-        # for 1. port schedule untouched, add only 3'rd chassis
-        # for 2. port master scheduler somewhere else
-        # for 3. port schedule all
-        self.mock_schedule.side_effect = [
-            ['chassis1', 'chassis2', 'chassis3'],
-            ['chassis1', 'chassis2', 'chassis3'],
-            ['chassis3', 'chassis2', 'chassis1']]
-
-        self.l3_inst.schedule_unhosted_gateways()
-
-        self.mock_candidates.assert_has_calls([
-            mock.call(mock.ANY,
-                      chassis_physnets=chassis_mappings,
-                      cms=chassis)] * 3)
-        self.mock_schedule.assert_has_calls([
-            mock.call(self.nb_idl(), self.sb_idl(),
-                      'lrp-foo-1', [], ['chassis1', 'chassis2']),
-            mock.call(self.nb_idl(), self.sb_idl(),
-                      'lrp-foo-2', [], ['chassis2']),
-            mock.call(self.nb_idl(), self.sb_idl(),
-                      'lrp-foo-3', [], [])])
-        # make sure that for second port master chassis stays untouched
-        self.nb_idl().update_lrouter_port.assert_has_calls([
-            mock.call('lrp-foo-1',
-                      gateway_chassis=['chassis1', 'chassis2', 'chassis3']),
-            mock.call('lrp-foo-2',
-                      gateway_chassis=['chassis2', 'chassis1', 'chassis3']),
-            mock.call('lrp-foo-3',
-                      gateway_chassis=['chassis3', 'chassis2', 'chassis1'])])
-
 
 class OVNL3ExtrarouteTests(test_l3_gw.ExtGwModeIntTestCase,
                            test_l3.L3NatDBIntTestCase,
