@@ -11,9 +11,8 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 #
-import collections
+
 import copy
-import uuid
 
 import mock
 
@@ -583,12 +582,19 @@ class TestNBImplIdlOvn(TestDBImplIdlOvn):
         # Test only host-1 in the valid list
         unhosted_gateways = self.nb_ovn_idl.get_unhosted_gateways(
             {}, {'host-1': 'physnet1'}, [])
-        expected = ['lrp-orp-id-a1', 'lrp-orp-id-a2',
-                    'lrp-orp-id-a3', 'lrp-orp-id-b2']
+        expected = {
+            utils.ovn_lrouter_port_name('orp-id-b2'): {
+                ovn_const.OVN_GATEWAY_CHASSIS_KEY: 'host-2'},
+            utils.ovn_lrouter_port_name('orp-id-a3'): {
+                ovn_const.OVN_GATEWAY_CHASSIS_KEY:
+                    ovn_const.OVN_GATEWAY_INVALID_CHASSIS}}
         self.assertItemsEqual(unhosted_gateways, expected)
         # Test both host-1, host-2 in valid list
         unhosted_gateways = self.nb_ovn_idl.get_unhosted_gateways(
             {}, {'host-1': 'physnet1', 'host-2': 'physnet2'}, [])
+        expected = {utils.ovn_lrouter_port_name('orp-id-a3'): {
+            ovn_const.OVN_GATEWAY_CHASSIS_KEY:
+                ovn_const.OVN_GATEWAY_INVALID_CHASSIS}}
         self.assertItemsEqual(unhosted_gateways, expected)
         # Schedule unhosted_gateways on host-2
         for unhosted_gateway in unhosted_gateways:
@@ -598,25 +604,7 @@ class TestNBImplIdlOvn(TestDBImplIdlOvn):
                 ovn_const.OVN_GATEWAY_CHASSIS_KEY: 'host-2'})
         unhosted_gateways = self.nb_ovn_idl.get_unhosted_gateways(
             {}, {'host-1': 'physnet1', 'host-2': 'physnet2'}, [])
-        self.assertItemsEqual(unhosted_gateways, expected)
-
-    def test_unhosted_gateway_max_chassis(self):
-        gw_chassis_table = fakes.FakeOvsdbTable.create_one_ovsdb_table()
-        self._tables['Gateway_Chassis'] = gw_chassis_table
-        gw_chassis = collections.namedtuple('gw_chassis',
-                                            'chassis_name priority')
-        TestNBImplIdlOvn.fake_set['lrouter_ports'][0]['gateway_chassis'] = [
-            gw_chassis(chassis_name='host-%s' % x,
-                       priority=x) for x in range(1, 6)]
-        for port in TestNBImplIdlOvn.fake_set['lrouter_ports'][1:]:
-            port['gateway_chassis'] = []
-        self._load_nb_db()
-        unhosted_gateways = self.nb_ovn_idl.get_unhosted_gateways(
-            {}, {'host-1': 'physnet1', 'host-2': 'physnet2',
-                 'host-3': 'physnet1', 'host-4': 'physnet2',
-                 'host-5': 'physnet1', 'host-6': 'physnet2'}, [])
-        expected = []
-        self.assertItemsEqual(unhosted_gateways, expected)
+        self.assertItemsEqual(unhosted_gateways, {})
 
     def test_get_subnet_dhcp_options(self):
         self._load_nb_db()
@@ -708,20 +696,6 @@ class TestNBImplIdlOvn(TestDBImplIdlOvn):
         address_sets = self.nb_ovn_idl.get_address_sets()
         self.assertEqual(len(address_sets), 4)
 
-    def test_get_port_group_not_supported(self):
-        self._load_nb_db()
-        # Make sure that PG tables doesn't exist in fake db.
-        self._tables.pop('Port_Group', None)
-        port_group = self.nb_ovn_idl.get_port_group(str(uuid.uuid4()))
-        self.assertIsNone(port_group)
-
-    def test_get_port_groups_not_supported(self):
-        self._load_nb_db()
-        # Make sure that PG tables doesn't exist in fake db.
-        self._tables.pop('Port_Group', None)
-        port_groups = self.nb_ovn_idl.get_port_groups()
-        self.assertEqual({}, port_groups)
-
 
 class TestSBImplIdlOvn(TestDBImplIdlOvn):
 
